@@ -344,6 +344,17 @@ sub build_vpaned {
         1;
 }
 
+sub build_hpaned {
+	my $self = shift;
+	my ($hpaned) = @_;
+        
+        my $gtk_hpaned = Gtk2::HPaned->new;
+        $gtk_hpaned->set ( position_set => 1 );
+        $hpaned->set_gtk_widget($gtk_hpaned);
+        
+        1;
+}
+
 sub build_vbox {
 	my $self = shift;
 	my ($vbox) = @_;
@@ -389,7 +400,7 @@ sub build_label {
 	my ($label) = @_;
 	
 	my $gtk_label = Gtk2::Label->new;
-	$gtk_label->set_text   ($label->get_label) if $label->get_label;
+	$gtk_label->set_text   ($label->get_label) if $label->get_label && !$label->get_attr;
 	$gtk_label->set_markup ($label->get_label) if $label->get_with_markup;
 	$gtk_label->set_markup ("<b>".$label->get_label."</b>")
 		if $label->get_label && $label->get_bold;
@@ -532,15 +543,21 @@ sub build_button {
 	my ($button, $gtk_button_class) = @_;
 
         $gtk_button_class ||= "Gtk2::Button";
+
 	my $stock = $button->get_stock;
 	my $label = $button->get_label;
+        my $image = $button->get_image;
 
 	my $gtk_button;
 
-# print "$gtk_button_class => '$label' ".(defined $label ? "DEF" : "UNDEF")."\n";;
-
+        #-- Button with image
+        if ( $image ) {
+                my $image = Gtk2::Image->new_from_file($image);
+		$gtk_button = $gtk_button_class->new;
+		$gtk_button->add($image);
+        }
         #-- Button with stock and text
-	if ( $stock and $label ne '' ) {
+        elsif ( $stock and $label ne '' ) {
 		my $hbox = Gtk2::HBox->new;
 		my $image = Gtk2::Image->new_from_stock($stock,"small-toolbar");
 		my $gtk_label = Gtk2::Label->new($label);
@@ -570,10 +587,33 @@ sub build_button {
 
 	$button->set_gtk_widget($gtk_button);
 	
+        my $with_repeat  = $button->can("get_with_repeat") && $button->get_with_repeat;
 	my $clicked_hook = $button->get_clicked_hook;
-	
-	$gtk_button->signal_connect ( clicked => sub { $clicked_hook->($button) } )
-		if $clicked_hook;
+        
+        if ( $clicked_hook && !$with_repeat ) {
+            $gtk_button->signal_connect ( clicked => sub { $clicked_hook->($button) } );
+        }
+
+        if ( $clicked_hook && $with_repeat ) {
+            my $button_pressed;
+            my $timeout;
+            $gtk_button->signal_connect ( button_press_event => sub {
+                $clicked_hook->($button);
+                $button_pressed = 1;
+                $timeout = Glib::Timeout->add ( 80, sub {
+                    return 0 if $button_pressed == 0;
+                    ++$button_pressed;
+                    $clicked_hook->($button) if $button_pressed > 5;
+                    return 1;
+                } );
+                0;
+            });
+            $gtk_button->signal_connect ( button_release_event => sub {
+                Glib::Source->remove($timeout);
+                $button_pressed = 0;
+                0;
+            });
+        }
 	
 	1;
 }
@@ -851,10 +891,18 @@ sub add_widget_to_form {
 	my $xopt = $widget->get_expand_h ? ['fill','expand'] : ['fill'];
 	my $yopt = $widget->get_expand_v ? ['fill','expand'] : ['fill'];
 
-	if ( $widget->get_label ne '' and not $widget->has_label ) {
+        if ( (! defined $widget->get_label || $widget->get_label ne '') and not $widget->has_label ) {
 		my $gtk_label = $self->create_label_widget ($widget);
-		$gtk_table->attach($gtk_label, 0, 1, $row, $row+1, 'fill', 'fill', 0, 0);
+
 		$widget->set_gtk_label_widget ($gtk_label);
+
+                if ( $form->get_label_top_align ) {
+                    my $gtk_align = Gtk2::Alignment->new(0, 0, 0, 0);
+                    $gtk_align->add($gtk_label);
+                    $gtk_label = $gtk_align;
+                }
+                
+		$gtk_table->attach($gtk_label, 0, 1, $row, $row+1, 'fill', 'fill', 0, 0);
 	}
 
 	if ( not $widget->get_expand_h ) {
@@ -880,7 +928,7 @@ sub add_widget_to_form_label_right {
 
 	$gtk_table->attach_defaults($gtk_entry, 0, 1, $row, $row+1);
 
-	if ( $widget->get_label ne '' and not $widget->has_label ) {
+        if ( (! defined $widget->get_label || $widget->get_label ne '') and not $widget->has_label ) {
 		my $gtk_label = $self->create_label_widget ($widget);
 		$gtk_table->attach($gtk_label, 1, 2, $row, $row+1, 'fill', [], 0, 0);
 		$widget->set_gtk_label_widget ($gtk_label);
@@ -943,6 +991,12 @@ sub add_widget_to_vpaned {
         1;
 }
 
+sub add_widget_to_hpaned {
+        my $self = shift;
+        my ($widget, $hpaned) = @_;
+        return $self->add_widget_to_vpaned($widget, $hpaned);
+}
+
 sub add_widget_to_vbox {
 	my $self = shift;
 	my ($widget, $vbox) = @_;
